<?php

namespace Drupal\entityreference_extensions;

use Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem;

/**
 * Trait EntityReferenceDeltaFilterTrait.
 *
 * Provides a filter callback and other helpers that are used by
 * multiple classes that do not and can not inherit from the same parent.
 *
 * @package Drupal\entityreference_extensions
 */
trait EntityReferenceDeltaFilterTrait {

  /**
   * Function that checks whether and item should pass the filter or not.
   *
   * @param \Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem $item
   *   An entity reference item.
   *
   * @return bool
   *   true if element passes the filter, false otherwise
   */
  public function deltaFilter(EntityReferenceItem $item) {
    $delta = intval($item->getName());
    $offset = intval($this->getSetting('offset'));
    if ($this->getSetting('reverse')) {
      if ($delta > ($this->noItems - 1 - $offset)) {
        return FALSE;
      }
      if ($delta >= ($this->noItems - 1 - $offset) - $this->getSetting('delta')) {
        return TRUE;
      }
      else {
        return FALSE;
      }
    }
    else {
      if ($delta > $this->getSetting('delta') + $offset) {
        return FALSE;
      }
      if ($delta < $offset) {
        return FALSE;
      }
      return TRUE;
    }
  }

  /**
   * Check whether field cardinality is not 1 (detect multi value field).
   *
   * @return bool
   *   true if multi value field
   */
  public function fieldCardinalityValid() {
    if ($this->fieldDefinition->getFieldStorageDefinition()->getCardinality() != 1) {
      return TRUE;
    }
    else {
      return FALSE;
    }
  }

  /**
   * Provide the setting form element for limiting entity reference items.
   *
   * @return array
   *   form element
   */
  public function deltaSetting() {
    $options = [
      'all' => $this->t('All'),
    ];
    for ($i = 0; $i < $this->getCardinalityCounter(); $i++) {
      $options[$i] = $i + 1;
    }
    return [
      '#type' => 'select',
      '#title' => $this->t('Limit elements'),
      '#options' => $options,
      '#default_value' => $this->getSetting('delta'),
    ];
  }

  /**
   * Provide the setting form element for reverse order of filtered items.
   *
   * @return array
   *   form element
   */
  public function reverseSetting() {
    return [
      '#type' => 'checkbox',
      '#title' => $this->t('Reverse order'),
      '#description' => $this->t('Use this if you want the last item, last two items, etc.'),
      '#default_value' => $this->getSetting('reverse'),
    ];
  }

  /**
   * Provide the setting form element for offsetting of filtered items.
   *
   * @return array
   *   form element
   */
  public function offsetSetting() {
    $options = [
      '0' => $this->t('None'),
    ];
    for ($i = 1; $i < $this->getCardinalityCounter(); $i++) {
      $options[$i] = $i;
    }
    return [
      '#type' => 'select',
      '#title' => $this->t('Offset'),
      '#options' => $options,
      '#default_value' => $this->getSetting('offset'),
    ];
  }

  /**
   * Get the cardinality counter for select options.
   *
   * Get cardinality of the field or for unlimited cardinality
   * get a configured max value used for select options.
   *
   * @return int
   *   cardinality counter number
   */
  public function getCardinalityCounter() {
    if ($this->fieldDefinition->getFieldStorageDefinition()->getCardinality() == -1) {
      return \Drupal::config('entityreference_extensions.settings')->get('unlimitedcounter') ? \Drupal::config('entityreference_extensions.settings')->get('unlimitedcounter') : 10;
    }
    else {
      return $this->fieldDefinition->getFieldStorageDefinition()->getCardinality();
    }
  }

}
